/* A Bison parser, made by GNU Bison 2.3.  */

/* Skeleton implementation for Bison's Yacc-like parsers in C

   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
   Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.

   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* C LALR(1) parser skeleton written by Richard Stallman, by
   simplifying the original so-called "semantic" parser.  */

/* All symbols defined below should begin with yy or YY, to avoid
   infringing on user name space.  This should be done even for local
   variables, as they might otherwise be expanded by user macros.
   There are some unavoidable exceptions within include files to
   define necessary library symbols; they are noted "INFRINGES ON
   USER NAME SPACE" below.  */

/* Identify Bison output.  */
#define YYBISON 1

/* Bison version.  */
#define YYBISON_VERSION "2.3"

/* Skeleton name.  */
#define YYSKELETON_NAME "yacc.c"

/* Pure parsers.  */
#define YYPURE 0

/* Using locations.  */
#define YYLSP_NEEDED 0



/* Tokens.  */
#ifndef YYTOKENTYPE
# define YYTOKENTYPE
   /* Put the tokens into the symbol table, so that GDB and other debuggers
      know about them.  */
   enum yytokentype {
     NUMBER = 258,
     VARIABLE = 259,
     IN = 260,
     DO = 261,
     WHILE = 262,
     ELSE = 263,
     RANGE = 264,
     EQ = 265,
     NE = 266,
     LT = 267,
     GT = 268,
     LE = 269,
     GE = 270,
     SUM = 271,
     PROD = 272,
     COUNT = 273,
     LOW = 274,
     HIGH = 275,
     DICE = 276,
     PERM = 277,
     SORT = 278,
     REV = 279,
     DROP = 280,
     KEEP = 281,
     FIRST = 282,
     LAST = 283,
     LET = 284,
     FOREACH = 285,
     IF = 286,
     THEN = 287
   };
#endif
/* Tokens.  */
#define NUMBER 258
#define VARIABLE 259
#define IN 260
#define DO 261
#define WHILE 262
#define ELSE 263
#define RANGE 264
#define EQ 265
#define NE 266
#define LT 267
#define GT 268
#define LE 269
#define GE 270
#define SUM 271
#define PROD 272
#define COUNT 273
#define LOW 274
#define HIGH 275
#define DICE 276
#define PERM 277
#define SORT 278
#define REV 279
#define DROP 280
#define KEEP 281
#define FIRST 282
#define LAST 283
#define LET 284
#define FOREACH 285
#define IF 286
#define THEN 287




/* Copy the first part of user declarations.  */
#line 1 "par.y"

#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

#include "util.h"
#include "tree.h"

int yylex();
int yyerror(char *s);
int yyget_lineno();

char *filename = "<stdin>";

extern int cmd_parse_tree;
extern int cmd_roll;
extern int cmd_rolleval;
extern int cmd_calc;
extern int cmd_count;

int cmplist(const void *p1, const void *p2) {
	return ((struct val_list*)p1)->values[0] 
        - ((struct val_list*)p2)->values[0];
}

/*
XXX use qsort from libc, but for some reason this segfaults
void list_quicksort(struct val_list **data, int start, int end) {
	qsort(&data[start], end-start+1, sizeof(struct val_list*), cmplist);
}*/

void list_quicksort(struct val_list **data, int start, int end) {
	if (end > start) {
		int i=start-1;
		int j = end;
		struct val_list *t;
		for(;;) {
            while(cmplist(data[++i], data[end]) < 0);
            while(cmplist(data[--j >= 0 ? j : end], data[end]) > 0);
			if (i>=j)
				break;
			t = data[i];
			data[i] = data[j];
			data[j] = t;

		}
		t = data[i];
		data[i] = data[end];
		data[end] = t;

		list_quicksort(data, start, i-1);
		list_quicksort(data, i+1, end);
	}
}

struct val_list *list_sort(struct val_list *list) {
	int count = 0;
	struct val_list *cl = list;
	// count the list
	while (cl) {
		count++;
		cl = cl->next;
	}
	// create an array
	struct val_list **hash = (struct val_list**)malloc(
        sizeof(struct val_list*) * count);
	int i = 0;
	cl = list;
	while (cl) {
		hash[i] = cl;
		cl = cl->next;
		i++;
	}
	// now sort the hash
	list_quicksort(hash, 0, count-1);
	for (i = 0; i < count-1; i++) {
		hash[i]->next = hash[i+1];
	}
	hash[count-1]->next = NULL;
	cl = hash[0];
	free(hash);
	return cl;
}



/* Enabling traces.  */
#ifndef YYDEBUG
# define YYDEBUG 0
#endif

/* Enabling verbose error messages.  */
#ifdef YYERROR_VERBOSE
# undef YYERROR_VERBOSE
# define YYERROR_VERBOSE 1
#else
# define YYERROR_VERBOSE 1
#endif

/* Enabling the token table.  */
#ifndef YYTOKEN_TABLE
# define YYTOKEN_TABLE 0
#endif

#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
typedef union YYSTYPE
#line 88 "par.y"
{
	int ival;
	char *tval;
	expression *expr;
}
/* Line 187 of yacc.c.  */
#line 252 "par.c"
	YYSTYPE;
# define yystype YYSTYPE /* obsolescent; will be withdrawn */
# define YYSTYPE_IS_DECLARED 1
# define YYSTYPE_IS_TRIVIAL 1
#endif



/* Copy the second part of user declarations.  */


/* Line 216 of yacc.c.  */
#line 265 "par.c"

#ifdef short
# undef short
#endif

#ifdef YYTYPE_UINT8
typedef YYTYPE_UINT8 yytype_uint8;
#else
typedef unsigned char yytype_uint8;
#endif

#ifdef YYTYPE_INT8
typedef YYTYPE_INT8 yytype_int8;
#elif (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
typedef signed char yytype_int8;
#else
typedef short int yytype_int8;
#endif

#ifdef YYTYPE_UINT16
typedef YYTYPE_UINT16 yytype_uint16;
#else
typedef unsigned short int yytype_uint16;
#endif

#ifdef YYTYPE_INT16
typedef YYTYPE_INT16 yytype_int16;
#else
typedef short int yytype_int16;
#endif

#ifndef YYSIZE_T
# ifdef __SIZE_TYPE__
#  define YYSIZE_T __SIZE_TYPE__
# elif defined size_t
#  define YYSIZE_T size_t
# elif ! defined YYSIZE_T && (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
#  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  define YYSIZE_T size_t
# else
#  define YYSIZE_T unsigned int
# endif
#endif

#define YYSIZE_MAXIMUM ((YYSIZE_T) -1)

#ifndef YY_
# if YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
#   define YY_(msgid) dgettext ("bison-runtime", msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(msgid) msgid
# endif
#endif

/* Suppress unused-variable warnings by "using" E.  */
#if ! defined lint || defined __GNUC__
# define YYUSE(e) ((void) (e))
#else
# define YYUSE(e) /* empty */
#endif

/* Identity function, used to suppress warnings about constant conditions.  */
#ifndef lint
# define YYID(n) (n)
#else
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static int
YYID (int i)
#else
static int
YYID (i)
    int i;
#endif
{
  return i;
}
#endif

#if ! defined yyoverflow || YYERROR_VERBOSE

/* The parser invokes alloca or malloc; define the necessary symbols.  */

# ifdef YYSTACK_USE_ALLOCA
#  if YYSTACK_USE_ALLOCA
#   ifdef __GNUC__
#    define YYSTACK_ALLOC __builtin_alloca
#   elif defined __BUILTIN_VA_ARG_INCR
#    include <alloca.h> /* INFRINGES ON USER NAME SPACE */
#   elif defined _AIX
#    define YYSTACK_ALLOC __alloca
#   elif defined _MSC_VER
#    include <malloc.h> /* INFRINGES ON USER NAME SPACE */
#    define alloca _alloca
#   else
#    define YYSTACK_ALLOC alloca
#    if ! defined _ALLOCA_H && ! defined _STDLIB_H && (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
#     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
#     ifndef _STDLIB_H
#      define _STDLIB_H 1
#     endif
#    endif
#   endif
#  endif
# endif

# ifdef YYSTACK_ALLOC
   /* Pacify GCC's `empty if-body' warning.  */
#  define YYSTACK_FREE(Ptr) do { /* empty */; } while (YYID (0))
#  ifndef YYSTACK_ALLOC_MAXIMUM
    /* The OS might guarantee only one guard page at the bottom of the stack,
       and a page size can be as small as 4096 bytes.  So we cannot safely
       invoke alloca (N) if N exceeds 4096.  Use a slightly smaller number
       to allow for a few compiler-allocated temporary stack slots.  */
#   define YYSTACK_ALLOC_MAXIMUM 4032 /* reasonable circa 2006 */
#  endif
# else
#  define YYSTACK_ALLOC YYMALLOC
#  define YYSTACK_FREE YYFREE
#  ifndef YYSTACK_ALLOC_MAXIMUM
#   define YYSTACK_ALLOC_MAXIMUM YYSIZE_MAXIMUM
#  endif
#  if (defined __cplusplus && ! defined _STDLIB_H \
       && ! ((defined YYMALLOC || defined malloc) \
	     && (defined YYFREE || defined free)))
#   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
#   ifndef _STDLIB_H
#    define _STDLIB_H 1
#   endif
#  endif
#  ifndef YYMALLOC
#   define YYMALLOC malloc
#   if ! defined malloc && ! defined _STDLIB_H && (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
#  ifndef YYFREE
#   define YYFREE free
#   if ! defined free && ! defined _STDLIB_H && (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
void free (void *); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
# endif
#endif /* ! defined yyoverflow || YYERROR_VERBOSE */


#if (! defined yyoverflow \
     && (! defined __cplusplus \
	 || (defined YYSTYPE_IS_TRIVIAL && YYSTYPE_IS_TRIVIAL)))

/* A type that is properly aligned for any stack member.  */
union yyalloc
{
  yytype_int16 yyss;
  YYSTYPE yyvs;
  };

/* The size of the maximum gap between one aligned stack and the next.  */
# define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)

/* The size of an array large to enough to hold all stacks, each with
   N elements.  */
# define YYSTACK_BYTES(N) \
     ((N) * (sizeof (yytype_int16) + sizeof (YYSTYPE)) \
      + YYSTACK_GAP_MAXIMUM)

/* Copy COUNT objects from FROM to TO.  The source and destination do
   not overlap.  */
# ifndef YYCOPY
#  if defined __GNUC__ && 1 < __GNUC__
#   define YYCOPY(To, From, Count) \
      __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
#  else
#   define YYCOPY(To, From, Count)		\
      do					\
	{					\
	  YYSIZE_T yyi;				\
	  for (yyi = 0; yyi < (Count); yyi++)	\
	    (To)[yyi] = (From)[yyi];		\
	}					\
      while (YYID (0))
#  endif
# endif

/* Relocate STACK from its old location to the new one.  The
   local variables YYSIZE and YYSTACKSIZE give the old and new number of
   elements in the stack, and YYPTR gives the new location of the
   stack.  Advance YYPTR to a properly aligned location for the next
   stack.  */
# define YYSTACK_RELOCATE(Stack)					\
    do									\
      {									\
	YYSIZE_T yynewbytes;						\
	YYCOPY (&yyptr->Stack, Stack, yysize);				\
	Stack = &yyptr->Stack;						\
	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
	yyptr += yynewbytes / sizeof (*yyptr);				\
      }									\
    while (YYID (0))

#endif

/* YYFINAL -- State number of the termination state.  */
#define YYFINAL  81
/* YYLAST -- Last index in YYTABLE.  */
#define YYLAST   349

/* YYNTOKENS -- Number of terminals.  */
#define YYNTOKENS  46
/* YYNNTS -- Number of nonterminals.  */
#define YYNNTS  7
/* YYNRULES -- Number of rules.  */
#define YYNRULES  65
/* YYNRULES -- Number of states.  */
#define YYNSTATES  173

/* YYTRANSLATE(YYLEX) -- Bison symbol number corresponding to YYLEX.  */
#define YYUNDEFTOK  2
#define YYMAXUTOK   287

#define YYTRANSLATE(YYX)						\
  ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)

/* YYTRANSLATE[YYLEX] -- Bison symbol number corresponding to YYLEX.  */
static const yytype_uint8 yytranslate[] =
{
       0,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,    28,     2,    22,     2,     2,
      43,    44,    20,    17,    10,    18,    19,    21,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,    42,
       2,    45,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,    29,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     1,     2,     3,     4,
       5,     6,     7,     8,     9,    11,    12,    13,    14,    15,
      16,    23,    24,    25,    26,    27,    30,    31,    32,    33,
      34,    35,    36,    37,    38,    39,    40,    41
};

#if YYDEBUG
/* YYPRHS[YYN] -- Index of the first RHS symbol of rule number YYN in
   YYRHS.  */
static const yytype_uint16 yyprhs[] =
{
       0,     0,     3,     5,     9,    12,    14,    16,    18,    20,
      22,    26,    29,    33,    37,    41,    45,    49,    53,    57,
      60,    63,    66,    69,    73,    77,    80,    84,    88,    91,
      94,    97,    99,   106,   113,   120,   127,   132,   137,   141,
     146,   151,   155,   160,   165,   169,   174,   179,   183,   188,
     193,   197,   202,   207,   211,   216,   221,   225,   230,   235,
     239,   244,   249,   253,   258,   263
};

/* YYRHS -- A `-1'-separated list of the rules' RHS.  */
static const yytype_int8 yyrhs[] =
{
      47,     0,    -1,    48,    -1,    47,    42,    48,    -1,    47,
      42,    -1,    49,    -1,    50,    -1,    51,    -1,     3,    -1,
       4,    -1,    43,    50,    44,    -1,    18,    50,    -1,    50,
      17,    50,    -1,    50,    18,    50,    -1,    50,    20,    50,
      -1,    50,    21,    50,    -1,    50,    22,    50,    -1,    50,
      29,    50,    -1,    50,    19,    50,    -1,    30,    50,    -1,
      23,    49,    -1,    24,    49,    -1,    25,    49,    -1,    50,
      28,    49,    -1,    43,    51,    44,    -1,    43,    44,    -1,
      50,     9,    50,    -1,    49,    10,    49,    -1,    31,    49,
      -1,    32,    49,    -1,    33,    49,    -1,    52,    -1,    40,
      49,    41,    49,     8,    49,    -1,    38,     4,    45,    49,
       5,    49,    -1,     7,     4,    45,    49,     6,    49,    -1,
      39,     4,     5,    49,     6,    49,    -1,    34,    36,    49,
      49,    -1,    35,    36,    49,    49,    -1,    36,    49,    49,
      -1,    34,    37,    49,    49,    -1,    35,    37,    49,    49,
      -1,    37,    49,    49,    -1,    34,    27,    49,    49,    -1,
      35,    27,    49,    49,    -1,    27,    49,    49,    -1,    34,
      26,    49,    49,    -1,    35,    26,    49,    49,    -1,    26,
      49,    49,    -1,    34,    11,    49,    49,    -1,    35,    11,
      49,    49,    -1,    11,    49,    49,    -1,    34,    12,    49,
      49,    -1,    35,    12,    49,    49,    -1,    12,    49,    49,
      -1,    34,    14,    49,    49,    -1,    35,    14,    49,    49,
      -1,    14,    49,    49,    -1,    34,    13,    49,    49,    -1,
      35,    13,    49,    49,    -1,    13,    49,    49,    -1,    34,
      16,    49,    49,    -1,    35,    16,    49,    49,    -1,    16,
      49,    49,    -1,    34,    15,    49,    49,    -1,    35,    15,
      49,    49,    -1,    15,    49,    49,    -1
};

/* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
static const yytype_uint16 yyrline[] =
{
       0,   117,   117,   118,   119,   122,   193,   194,   197,   198,
     199,   200,   201,   202,   203,   204,   205,   206,   207,   208,
     209,   210,   211,   214,   215,   216,   217,   218,   219,   220,
     221,   222,   223,   224,   225,   226,   229,   230,   231,   232,
     233,   234,   235,   236,   237,   238,   239,   240,   241,   242,
     243,   244,   245,   246,   247,   248,   249,   250,   251,   252,
     253,   254,   255,   256,   257,   258
};
#endif

#if YYDEBUG || YYERROR_VERBOSE || YYTOKEN_TABLE
/* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
   First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
static const char *const yytname[] =
{
  "$end", "error", "$undefined", "NUMBER", "VARIABLE", "IN", "DO",
  "WHILE", "ELSE", "RANGE", "','", "EQ", "NE", "LT", "GT", "LE", "GE",
  "'+'", "'-'", "'.'", "'*'", "'/'", "'%'", "SUM", "PROD", "COUNT", "LOW",
  "HIGH", "'#'", "'^'", "DICE", "PERM", "SORT", "REV", "DROP", "KEEP",
  "FIRST", "LAST", "LET", "FOREACH", "IF", "THEN", "';'", "'('", "')'",
  "'='", "$accept", "input", "pexpr", "expr", "scalar", "list", "filter", 0
};
#endif

# ifdef YYPRINT
/* YYTOKNUM[YYLEX-NUM] -- Internal token number corresponding to
   token YYLEX-NUM.  */
static const yytype_uint16 yytoknum[] =
{
       0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
      44,   265,   266,   267,   268,   269,   270,    43,    45,    46,
      42,    47,    37,   271,   272,   273,   274,   275,    35,    94,
     276,   277,   278,   279,   280,   281,   282,   283,   284,   285,
     286,   287,    59,    40,    41,    61
};
# endif

/* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
static const yytype_uint8 yyr1[] =
{
       0,    46,    47,    47,    47,    48,    49,    49,    50,    50,
      50,    50,    50,    50,    50,    50,    50,    50,    50,    50,
      50,    50,    50,    51,    51,    51,    51,    51,    51,    51,
      51,    51,    51,    51,    51,    51,    52,    52,    52,    52,
      52,    52,    52,    52,    52,    52,    52,    52,    52,    52,
      52,    52,    52,    52,    52,    52,    52,    52,    52,    52,
      52,    52,    52,    52,    52,    52
};

/* YYR2[YYN] -- Number of symbols composing right hand side of rule YYN.  */
static const yytype_uint8 yyr2[] =
{
       0,     2,     1,     3,     2,     1,     1,     1,     1,     1,
       3,     2,     3,     3,     3,     3,     3,     3,     3,     2,
       2,     2,     2,     3,     3,     2,     3,     3,     2,     2,
       2,     1,     6,     6,     6,     6,     4,     4,     3,     4,
       4,     3,     4,     4,     3,     4,     4,     3,     4,     4,
       3,     4,     4,     3,     4,     4,     3,     4,     4,     3,
       4,     4,     3,     4,     4,     3
};

/* YYDEFACT[STATE-NAME] -- Default rule to reduce with in state
   STATE-NUM when YYTABLE doesn't specify something else to do.  Zero
   means the default is an error.  */
static const yytype_uint8 yydefact[] =
{
       0,     8,     9,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     2,
       5,     6,     7,    31,     0,     0,     0,     0,     0,     0,
       0,     0,    11,    20,    21,    22,     0,     0,    19,    28,
      29,    30,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    25,     0,     6,
       7,     1,     4,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,    50,    53,    59,    56,    65,    62,
       0,    47,    44,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,    38,    41,     0,     0,     0,    10,    24,
       3,    27,    26,    12,    13,    18,    14,    15,    16,    23,
      17,     0,    48,    51,    57,    54,    63,    60,    45,    42,
      36,    39,    49,    52,    58,    55,    64,    61,    46,    43,
      37,    40,     0,     0,     0,     0,     0,     0,     0,    34,
      33,    35,    32
};

/* YYDEFGOTO[NTERM-NUM].  */
static const yytype_int8 yydefgoto[] =
{
      -1,    28,    29,    30,    31,    32,    33
};

/* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
   STATE-NUM.  */
#define YYPACT_NINF -5
static const yytype_int16 yypact[] =
{
     237,    -5,    -5,     2,   237,   237,   237,   237,   237,   237,
     275,   237,   237,   237,   237,   237,   275,   237,   237,   237,
     115,   270,   237,   237,    22,    26,   237,   142,    28,    -5,
      30,   306,    -5,    -5,    -1,   199,   199,   199,   199,   199,
     199,   275,    17,    30,    30,    30,   199,   199,    -5,    30,
      30,    30,   237,   237,   237,   237,   237,   237,   237,   237,
     237,   237,   237,   237,   237,   237,   237,   237,   237,   237,
     237,   237,   199,   199,     0,    36,     6,    -5,    30,   292,
      33,    -5,   237,   237,   275,   275,   275,   275,   275,   275,
     275,   237,   275,   237,    30,    30,    30,    30,    30,    30,
      54,    30,    30,   199,   199,   199,   199,   199,   199,   199,
     199,   199,   199,   199,   199,   199,   199,   199,   199,   199,
     199,   199,   199,    30,    30,   237,   237,   237,    -5,    -5,
      -5,    30,   320,    17,    17,    17,    49,    49,    49,    -5,
      49,    14,    30,    30,    30,    30,    30,    30,    30,    30,
      30,    30,    30,    30,    30,    30,    30,    30,    30,    30,
      30,    30,     7,    15,    19,   237,   237,   237,   237,    30,
      30,    30,    30
};

/* YYPGOTO[NTERM-NUM].  */
static const yytype_int8 yypgoto[] =
{
      -5,    -5,    -2,    -4,   109,    55,    -5
};

/* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
   positive, shift that token.  If negative, reduce the rule which
   number is the opposite.  If zero, do what YYDEFACT says.
   If YYTABLE_NINF, syntax error.  */
#define YYTABLE_NINF -1
static const yytype_uint8 yytable[] =
{
      35,    36,    37,    38,    39,    40,    34,    43,    44,    45,
      46,    47,   166,    49,    50,    51,    83,    83,    72,    73,
     165,   167,    76,    78,    83,    83,    74,   168,    81,    83,
      75,    94,    95,    96,    97,    98,    99,    88,    89,    90,
      83,   126,   101,   102,    93,   125,    92,   127,   103,   104,
     105,   106,   107,   108,   109,   110,   111,   112,   113,   114,
     115,   116,   117,   118,   119,   120,   121,   122,   123,   124,
      82,    85,    86,    87,    88,    89,    90,   129,    92,   131,
     130,     0,    80,    92,     0,     0,     0,   139,     0,   141,
       0,     0,     0,     0,     0,     0,     0,     0,   128,   142,
     143,   144,   145,   146,   147,   148,   149,   150,   151,   152,
     153,   154,   155,   156,   157,   158,   159,   160,   161,    42,
       0,   162,   163,   164,     0,    48,    52,    53,    54,    55,
      56,    57,     0,     0,     0,     0,    79,     0,     0,     0,
       0,    58,    59,     0,     0,     1,     2,     0,     0,     3,
     100,    60,    61,     4,     5,     6,     7,     8,     9,     0,
      10,   169,   170,   171,   172,    11,    12,    13,    14,    15,
       0,     0,    16,    17,    18,    19,    20,    21,    22,    23,
      24,    25,    26,     0,     0,    27,    77,     0,     0,     0,
       0,     0,     0,   132,   133,   134,   135,   136,   137,   138,
       0,   140,     1,     2,     0,     0,     3,     0,     0,    83,
       4,     5,     6,     7,     8,     9,     0,    10,     0,     0,
       0,     0,    11,    12,    13,    14,    15,     0,     0,    16,
      17,    18,    19,    20,    21,    22,    23,    24,    25,    26,
       1,     2,    27,     0,     3,     0,     0,     0,     4,     5,
       6,     7,     8,     9,     0,    10,     0,     0,     0,     0,
      11,    12,    13,    14,    15,     0,     0,    16,    17,    18,
      19,    20,    21,    22,    23,    24,    25,    26,     1,     2,
      27,    62,    63,    64,    65,    66,    67,     0,     0,     0,
       0,     0,     0,    10,     0,     0,    68,    69,    11,    12,
      13,    84,     0,     0,     0,    16,    70,    71,     0,    85,
      86,    87,    88,    89,    90,    84,     0,     0,    41,     0,
      91,    92,     0,    85,    86,    87,    88,    89,    90,     0,
       0,     0,     0,     0,    91,    92,   128,    85,    86,    87,
      88,    89,    90,     0,     0,     0,     0,     0,     0,    92
};

static const yytype_int16 yycheck[] =
{
       4,     5,     6,     7,     8,     9,     4,    11,    12,    13,
      14,    15,     5,    17,    18,    19,    10,    10,    22,    23,
       6,     6,    26,    27,    10,    10,     4,     8,     0,    10,
       4,    35,    36,    37,    38,    39,    40,    20,    21,    22,
      10,     5,    46,    47,    45,    45,    29,    41,    52,    53,
      54,    55,    56,    57,    58,    59,    60,    61,    62,    63,
      64,    65,    66,    67,    68,    69,    70,    71,    72,    73,
      42,    17,    18,    19,    20,    21,    22,    44,    29,    83,
      82,    -1,    27,    29,    -1,    -1,    -1,    91,    -1,    93,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    44,   103,
     104,   105,   106,   107,   108,   109,   110,   111,   112,   113,
     114,   115,   116,   117,   118,   119,   120,   121,   122,    10,
      -1,   125,   126,   127,    -1,    16,    11,    12,    13,    14,
      15,    16,    -1,    -1,    -1,    -1,    27,    -1,    -1,    -1,
      -1,    26,    27,    -1,    -1,     3,     4,    -1,    -1,     7,
      41,    36,    37,    11,    12,    13,    14,    15,    16,    -1,
      18,   165,   166,   167,   168,    23,    24,    25,    26,    27,
      -1,    -1,    30,    31,    32,    33,    34,    35,    36,    37,
      38,    39,    40,    -1,    -1,    43,    44,    -1,    -1,    -1,
      -1,    -1,    -1,    84,    85,    86,    87,    88,    89,    90,
      -1,    92,     3,     4,    -1,    -1,     7,    -1,    -1,    10,
      11,    12,    13,    14,    15,    16,    -1,    18,    -1,    -1,
      -1,    -1,    23,    24,    25,    26,    27,    -1,    -1,    30,
      31,    32,    33,    34,    35,    36,    37,    38,    39,    40,
       3,     4,    43,    -1,     7,    -1,    -1,    -1,    11,    12,
      13,    14,    15,    16,    -1,    18,    -1,    -1,    -1,    -1,
      23,    24,    25,    26,    27,    -1,    -1,    30,    31,    32,
      33,    34,    35,    36,    37,    38,    39,    40,     3,     4,
      43,    11,    12,    13,    14,    15,    16,    -1,    -1,    -1,
      -1,    -1,    -1,    18,    -1,    -1,    26,    27,    23,    24,
      25,     9,    -1,    -1,    -1,    30,    36,    37,    -1,    17,
      18,    19,    20,    21,    22,     9,    -1,    -1,    43,    -1,
      28,    29,    -1,    17,    18,    19,    20,    21,    22,    -1,
      -1,    -1,    -1,    -1,    28,    29,    44,    17,    18,    19,
      20,    21,    22,    -1,    -1,    -1,    -1,    -1,    -1,    29
};

/* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
   symbol of state STATE-NUM.  */
static const yytype_uint8 yystos[] =
{
       0,     3,     4,     7,    11,    12,    13,    14,    15,    16,
      18,    23,    24,    25,    26,    27,    30,    31,    32,    33,
      34,    35,    36,    37,    38,    39,    40,    43,    47,    48,
      49,    50,    51,    52,     4,    49,    49,    49,    49,    49,
      49,    43,    50,    49,    49,    49,    49,    49,    50,    49,
      49,    49,    11,    12,    13,    14,    15,    16,    26,    27,
      36,    37,    11,    12,    13,    14,    15,    16,    26,    27,
      36,    37,    49,    49,     4,     4,    49,    44,    49,    50,
      51,     0,    42,    10,     9,    17,    18,    19,    20,    21,
      22,    28,    29,    45,    49,    49,    49,    49,    49,    49,
      50,    49,    49,    49,    49,    49,    49,    49,    49,    49,
      49,    49,    49,    49,    49,    49,    49,    49,    49,    49,
      49,    49,    49,    49,    49,    45,     5,    41,    44,    44,
      48,    49,    50,    50,    50,    50,    50,    50,    50,    49,
      50,    49,    49,    49,    49,    49,    49,    49,    49,    49,
      49,    49,    49,    49,    49,    49,    49,    49,    49,    49,
      49,    49,    49,    49,    49,     6,     5,     6,     8,    49,
      49,    49,    49
};

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		(-2)
#define YYEOF		0

#define YYACCEPT	goto yyacceptlab
#define YYABORT		goto yyabortlab
#define YYERROR		goto yyerrorlab


/* Like YYERROR except do call yyerror.  This remains here temporarily
   to ease the transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  */

#define YYFAIL		goto yyerrlab

#define YYRECOVERING()  (!!yyerrstatus)

#define YYBACKUP(Token, Value)					\
do								\
  if (yychar == YYEMPTY && yylen == 1)				\
    {								\
      yychar = (Token);						\
      yylval = (Value);						\
      yytoken = YYTRANSLATE (yychar);				\
      YYPOPSTACK (1);						\
      goto yybackup;						\
    }								\
  else								\
    {								\
      yyerror (YY_("syntax error: cannot back up")); \
      YYERROR;							\
    }								\
while (YYID (0))


#define YYTERROR	1
#define YYERRCODE	256


/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
   If N is 0, then set CURRENT to the empty location which ends
   the previous symbol: RHS[0] (always defined).  */

#define YYRHSLOC(Rhs, K) ((Rhs)[K])
#ifndef YYLLOC_DEFAULT
# define YYLLOC_DEFAULT(Current, Rhs, N)				\
    do									\
      if (YYID (N))                                                    \
	{								\
	  (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;	\
	  (Current).first_column = YYRHSLOC (Rhs, 1).first_column;	\
	  (Current).last_line    = YYRHSLOC (Rhs, N).last_line;		\
	  (Current).last_column  = YYRHSLOC (Rhs, N).last_column;	\
	}								\
      else								\
	{								\
	  (Current).first_line   = (Current).last_line   =		\
	    YYRHSLOC (Rhs, 0).last_line;				\
	  (Current).first_column = (Current).last_column =		\
	    YYRHSLOC (Rhs, 0).last_column;				\
	}								\
    while (YYID (0))
#endif


/* YY_LOCATION_PRINT -- Print the location on the stream.
   This macro was not mandated originally: define only if we know
   we won't break user code: when these are the locations we know.  */

#ifndef YY_LOCATION_PRINT
# if YYLTYPE_IS_TRIVIAL
#  define YY_LOCATION_PRINT(File, Loc)			\
     fprintf (File, "%d.%d-%d.%d",			\
	      (Loc).first_line, (Loc).first_column,	\
	      (Loc).last_line,  (Loc).last_column)
# else
#  define YY_LOCATION_PRINT(File, Loc) ((void) 0)
# endif
#endif


/* YYLEX -- calling `yylex' with the right arguments.  */

#ifdef YYLEX_PARAM
# define YYLEX yylex (YYLEX_PARAM)
#else
# define YYLEX yylex ()
#endif

/* Enable debugging if requested.  */
#if YYDEBUG

# ifndef YYFPRINTF
#  include <stdio.h> /* INFRINGES ON USER NAME SPACE */
#  define YYFPRINTF fprintf
# endif

# define YYDPRINTF(Args)			\
do {						\
  if (yydebug)					\
    YYFPRINTF Args;				\
} while (YYID (0))

# define YY_SYMBOL_PRINT(Title, Type, Value, Location)			  \
do {									  \
  if (yydebug)								  \
    {									  \
      YYFPRINTF (stderr, "%s ", Title);					  \
      yy_symbol_print (stderr,						  \
		  Type, Value); \
      YYFPRINTF (stderr, "\n");						  \
    }									  \
} while (YYID (0))


/*--------------------------------.
| Print this symbol on YYOUTPUT.  |
`--------------------------------*/

/*ARGSUSED*/
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
#else
static void
yy_symbol_value_print (yyoutput, yytype, yyvaluep)
    FILE *yyoutput;
    int yytype;
    YYSTYPE const * const yyvaluep;
#endif
{
  if (!yyvaluep)
    return;
# ifdef YYPRINT
  if (yytype < YYNTOKENS)
    YYPRINT (yyoutput, yytoknum[yytype], *yyvaluep);
# else
  YYUSE (yyoutput);
# endif
  switch (yytype)
    {
      default:
	break;
    }
}


/*--------------------------------.
| Print this symbol on YYOUTPUT.  |
`--------------------------------*/

#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
#else
static void
yy_symbol_print (yyoutput, yytype, yyvaluep)
    FILE *yyoutput;
    int yytype;
    YYSTYPE const * const yyvaluep;
#endif
{
  if (yytype < YYNTOKENS)
    YYFPRINTF (yyoutput, "token %s (", yytname[yytype]);
  else
    YYFPRINTF (yyoutput, "nterm %s (", yytname[yytype]);

  yy_symbol_value_print (yyoutput, yytype, yyvaluep);
  YYFPRINTF (yyoutput, ")");
}

/*------------------------------------------------------------------.
| yy_stack_print -- Print the state stack from its BOTTOM up to its |
| TOP (included).                                                   |
`------------------------------------------------------------------*/

#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yy_stack_print (yytype_int16 *bottom, yytype_int16 *top)
#else
static void
yy_stack_print (bottom, top)
    yytype_int16 *bottom;
    yytype_int16 *top;
#endif
{
  YYFPRINTF (stderr, "Stack now");
  for (; bottom <= top; ++bottom)
    YYFPRINTF (stderr, " %d", *bottom);
  YYFPRINTF (stderr, "\n");
}

# define YY_STACK_PRINT(Bottom, Top)				\
do {								\
  if (yydebug)							\
    yy_stack_print ((Bottom), (Top));				\
} while (YYID (0))


/*------------------------------------------------.
| Report that the YYRULE is going to be reduced.  |
`------------------------------------------------*/

#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yy_reduce_print (YYSTYPE *yyvsp, int yyrule)
#else
static void
yy_reduce_print (yyvsp, yyrule)
    YYSTYPE *yyvsp;
    int yyrule;
#endif
{
  int yynrhs = yyr2[yyrule];
  int yyi;
  unsigned long int yylno = yyrline[yyrule];
  YYFPRINTF (stderr, "Reducing stack by rule %d (line %lu):\n",
	     yyrule - 1, yylno);
  /* The symbols being reduced.  */
  for (yyi = 0; yyi < yynrhs; yyi++)
    {
      fprintf (stderr, "   $%d = ", yyi + 1);
      yy_symbol_print (stderr, yyrhs[yyprhs[yyrule] + yyi],
		       &(yyvsp[(yyi + 1) - (yynrhs)])
		       		       );
      fprintf (stderr, "\n");
    }
}

# define YY_REDUCE_PRINT(Rule)		\
do {					\
  if (yydebug)				\
    yy_reduce_print (yyvsp, Rule); \
} while (YYID (0))

/* Nonzero means print parse trace.  It is left uninitialized so that
   multiple parsers can coexist.  */
int yydebug;
#else /* !YYDEBUG */
# define YYDPRINTF(Args)
# define YY_SYMBOL_PRINT(Title, Type, Value, Location)
# define YY_STACK_PRINT(Bottom, Top)
# define YY_REDUCE_PRINT(Rule)
#endif /* !YYDEBUG */


/* YYINITDEPTH -- initial size of the parser's stacks.  */
#ifndef	YYINITDEPTH
# define YYINITDEPTH 200
#endif

/* YYMAXDEPTH -- maximum size the stacks can grow to (effective only
   if the built-in stack extension method is used).

   Do not make this value too large; the results are undefined if
   YYSTACK_ALLOC_MAXIMUM < YYSTACK_BYTES (YYMAXDEPTH)
   evaluated with infinite-precision integer arithmetic.  */

#ifndef YYMAXDEPTH
# define YYMAXDEPTH 10000
#endif



#if YYERROR_VERBOSE

# ifndef yystrlen
#  if defined __GLIBC__ && defined _STRING_H
#   define yystrlen strlen
#  else
/* Return the length of YYSTR.  */
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static YYSIZE_T
yystrlen (const char *yystr)
#else
static YYSIZE_T
yystrlen (yystr)
    const char *yystr;
#endif
{
  YYSIZE_T yylen;
  for (yylen = 0; yystr[yylen]; yylen++)
    continue;
  return yylen;
}
#  endif
# endif

# ifndef yystpcpy
#  if defined __GLIBC__ && defined _STRING_H && defined _GNU_SOURCE
#   define yystpcpy stpcpy
#  else
/* Copy YYSRC to YYDEST, returning the address of the terminating '\0' in
   YYDEST.  */
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static char *
yystpcpy (char *yydest, const char *yysrc)
#else
static char *
yystpcpy (yydest, yysrc)
    char *yydest;
    const char *yysrc;
#endif
{
  char *yyd = yydest;
  const char *yys = yysrc;

  while ((*yyd++ = *yys++) != '\0')
    continue;

  return yyd - 1;
}
#  endif
# endif

# ifndef yytnamerr
/* Copy to YYRES the contents of YYSTR after stripping away unnecessary
   quotes and backslashes, so that it's suitable for yyerror.  The
   heuristic is that double-quoting is unnecessary unless the string
   contains an apostrophe, a comma, or backslash (other than
   backslash-backslash).  YYSTR is taken from yytname.  If YYRES is
   null, do not copy; instead, return the length of what the result
   would have been.  */
static YYSIZE_T
yytnamerr (char *yyres, const char *yystr)
{
  if (*yystr == '"')
    {
      YYSIZE_T yyn = 0;
      char const *yyp = yystr;

      for (;;)
	switch (*++yyp)
	  {
	  case '\'':
	  case ',':
	    goto do_not_strip_quotes;

	  case '\\':
	    if (*++yyp != '\\')
	      goto do_not_strip_quotes;
	    /* Fall through.  */
	  default:
	    if (yyres)
	      yyres[yyn] = *yyp;
	    yyn++;
	    break;

	  case '"':
	    if (yyres)
	      yyres[yyn] = '\0';
	    return yyn;
	  }
    do_not_strip_quotes: ;
    }

  if (! yyres)
    return yystrlen (yystr);

  return yystpcpy (yyres, yystr) - yyres;
}
# endif

/* Copy into YYRESULT an error message about the unexpected token
   YYCHAR while in state YYSTATE.  Return the number of bytes copied,
   including the terminating null byte.  If YYRESULT is null, do not
   copy anything; just return the number of bytes that would be
   copied.  As a special case, return 0 if an ordinary "syntax error"
   message will do.  Return YYSIZE_MAXIMUM if overflow occurs during
   size calculation.  */
static YYSIZE_T
yysyntax_error (char *yyresult, int yystate, int yychar)
{
  int yyn = yypact[yystate];

  if (! (YYPACT_NINF < yyn && yyn <= YYLAST))
    return 0;
  else
    {
      int yytype = YYTRANSLATE (yychar);
      YYSIZE_T yysize0 = yytnamerr (0, yytname[yytype]);
      YYSIZE_T yysize = yysize0;
      YYSIZE_T yysize1;
      int yysize_overflow = 0;
      enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
      char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
      int yyx;

# if 0
      /* This is so xgettext sees the translatable formats that are
	 constructed on the fly.  */
      YY_("syntax error, unexpected %s");
      YY_("syntax error, unexpected %s, expecting %s");
      YY_("syntax error, unexpected %s, expecting %s or %s");
      YY_("syntax error, unexpected %s, expecting %s or %s or %s");
      YY_("syntax error, unexpected %s, expecting %s or %s or %s or %s");
# endif
      char *yyfmt;
      char const *yyf;
      static char const yyunexpected[] = "syntax error, unexpected %s";
      static char const yyexpecting[] = ", expecting %s";
      static char const yyor[] = " or %s";
      char yyformat[sizeof yyunexpected
		    + sizeof yyexpecting - 1
		    + ((YYERROR_VERBOSE_ARGS_MAXIMUM - 2)
		       * (sizeof yyor - 1))];
      char const *yyprefix = yyexpecting;

      /* Start YYX at -YYN if negative to avoid negative indexes in
	 YYCHECK.  */
      int yyxbegin = yyn < 0 ? -yyn : 0;

      /* Stay within bounds of both yycheck and yytname.  */
      int yychecklim = YYLAST - yyn + 1;
      int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
      int yycount = 1;

      yyarg[0] = yytname[yytype];
      yyfmt = yystpcpy (yyformat, yyunexpected);

      for (yyx = yyxbegin; yyx < yyxend; ++yyx)
	if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
	  {
	    if (yycount == YYERROR_VERBOSE_ARGS_MAXIMUM)
	      {
		yycount = 1;
		yysize = yysize0;
		yyformat[sizeof yyunexpected - 1] = '\0';
		break;
	      }
	    yyarg[yycount++] = yytname[yyx];
	    yysize1 = yysize + yytnamerr (0, yytname[yyx]);
	    yysize_overflow |= (yysize1 < yysize);
	    yysize = yysize1;
	    yyfmt = yystpcpy (yyfmt, yyprefix);
	    yyprefix = yyor;
	  }

      yyf = YY_(yyformat);
      yysize1 = yysize + yystrlen (yyf);
      yysize_overflow |= (yysize1 < yysize);
      yysize = yysize1;

      if (yysize_overflow)
	return YYSIZE_MAXIMUM;

      if (yyresult)
	{
	  /* Avoid sprintf, as that infringes on the user's name space.
	     Don't have undefined behavior even if the translation
	     produced a string with the wrong number of "%s"s.  */
	  char *yyp = yyresult;
	  int yyi = 0;
	  while ((*yyp = *yyf) != '\0')
	    {
	      if (*yyp == '%' && yyf[1] == 's' && yyi < yycount)
		{
		  yyp += yytnamerr (yyp, yyarg[yyi++]);
		  yyf += 2;
		}
	      else
		{
		  yyp++;
		  yyf++;
		}
	    }
	}
      return yysize;
    }
}
#endif /* YYERROR_VERBOSE */


/*-----------------------------------------------.
| Release the memory associated to this symbol.  |
`-----------------------------------------------*/

/*ARGSUSED*/
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
static void
yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
#else
static void
yydestruct (yymsg, yytype, yyvaluep)
    const char *yymsg;
    int yytype;
    YYSTYPE *yyvaluep;
#endif
{
  YYUSE (yyvaluep);

  if (!yymsg)
    yymsg = "Deleting";
  YY_SYMBOL_PRINT (yymsg, yytype, yyvaluep, yylocationp);

  switch (yytype)
    {

      default:
	break;
    }
}


/* Prevent warnings from -Wmissing-prototypes.  */

#ifdef YYPARSE_PARAM
#if defined __STDC__ || defined __cplusplus
int yyparse (void *YYPARSE_PARAM);
#else
int yyparse ();
#endif
#else /* ! YYPARSE_PARAM */
#if defined __STDC__ || defined __cplusplus
int yyparse (void);
#else
int yyparse ();
#endif
#endif /* ! YYPARSE_PARAM */



/* The look-ahead symbol.  */
int yychar;

/* The semantic value of the look-ahead symbol.  */
YYSTYPE yylval;

/* Number of syntax errors so far.  */
int yynerrs;



/*----------.
| yyparse.  |
`----------*/

#ifdef YYPARSE_PARAM
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
int
yyparse (void *YYPARSE_PARAM)
#else
int
yyparse (YYPARSE_PARAM)
    void *YYPARSE_PARAM;
#endif
#else /* ! YYPARSE_PARAM */
#if (defined __STDC__ || defined __C99__FUNC__ \
     || defined __cplusplus || defined _MSC_VER)
int
yyparse (void)
#else
int
yyparse ()

#endif
#endif
{
  
  int yystate;
  int yyn;
  int yyresult;
  /* Number of tokens to shift before error messages enabled.  */
  int yyerrstatus;
  /* Look-ahead token as an internal (translated) token number.  */
  int yytoken = 0;
#if YYERROR_VERBOSE
  /* Buffer for error messages, and its allocated size.  */
  char yymsgbuf[128];
  char *yymsg = yymsgbuf;
  YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
#endif

  /* Three stacks and their tools:
     `yyss': related to states,
     `yyvs': related to semantic values,
     `yyls': related to locations.

     Refer to the stacks thru separate pointers, to allow yyoverflow
     to reallocate them elsewhere.  */

  /* The state stack.  */
  yytype_int16 yyssa[YYINITDEPTH];
  yytype_int16 *yyss = yyssa;
  yytype_int16 *yyssp;

  /* The semantic value stack.  */
  YYSTYPE yyvsa[YYINITDEPTH];
  YYSTYPE *yyvs = yyvsa;
  YYSTYPE *yyvsp;



#define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))

  YYSIZE_T yystacksize = YYINITDEPTH;

  /* The variables used to return semantic value and location from the
     action routines.  */
  YYSTYPE yyval;


  /* The number of symbols on the RHS of the reduced rule.
     Keep to zero when no symbol should be popped.  */
  int yylen = 0;

  YYDPRINTF ((stderr, "Starting parse\n"));

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY;		/* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.
     The wasted elements are never initialized.  */

  yyssp = yyss;
  yyvsp = yyvs;

  goto yysetstate;

/*------------------------------------------------------------.
| yynewstate -- Push a new state, which is found in yystate.  |
`------------------------------------------------------------*/
 yynewstate:
  /* In all cases, when you get here, the value and location stacks
     have just been pushed.  So pushing a state here evens the stacks.  */
  yyssp++;

 yysetstate:
  *yyssp = yystate;

  if (yyss + yystacksize - 1 <= yyssp)
    {
      /* Get the current used size of the three stacks, in elements.  */
      YYSIZE_T yysize = yyssp - yyss + 1;

#ifdef yyoverflow
      {
	/* Give user a chance to reallocate the stack.  Use copies of
	   these so that the &'s don't force the real ones into
	   memory.  */
	YYSTYPE *yyvs1 = yyvs;
	yytype_int16 *yyss1 = yyss;


	/* Each stack pointer address is followed by the size of the
	   data in use in that stack, in bytes.  This used to be a
	   conditional around just the two extra args, but that might
	   be undefined if yyoverflow is a macro.  */
	yyoverflow (YY_("memory exhausted"),
		    &yyss1, yysize * sizeof (*yyssp),
		    &yyvs1, yysize * sizeof (*yyvsp),

		    &yystacksize);

	yyss = yyss1;
	yyvs = yyvs1;
      }
#else /* no yyoverflow */
# ifndef YYSTACK_RELOCATE
      goto yyexhaustedlab;
# else
      /* Extend the stack our own way.  */
      if (YYMAXDEPTH <= yystacksize)
	goto yyexhaustedlab;
      yystacksize *= 2;
      if (YYMAXDEPTH < yystacksize)
	yystacksize = YYMAXDEPTH;

      {
	yytype_int16 *yyss1 = yyss;
	union yyalloc *yyptr =
	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
	if (! yyptr)
	  goto yyexhaustedlab;
	YYSTACK_RELOCATE (yyss);
	YYSTACK_RELOCATE (yyvs);

#  undef YYSTACK_RELOCATE
	if (yyss1 != yyssa)
	  YYSTACK_FREE (yyss1);
      }
# endif
#endif /* no yyoverflow */

      yyssp = yyss + yysize - 1;
      yyvsp = yyvs + yysize - 1;


      YYDPRINTF ((stderr, "Stack size increased to %lu\n",
		  (unsigned long int) yystacksize));

      if (yyss + yystacksize - 1 <= yyssp)
	YYABORT;
    }

  YYDPRINTF ((stderr, "Entering state %d\n", yystate));

  goto yybackup;

/*-----------.
| yybackup.  |
`-----------*/
yybackup:

  /* Do appropriate processing given the current state.  Read a
     look-ahead token if we need one and don't already have one.  */

  /* First try to decide what to do without reference to look-ahead token.  */
  yyn = yypact[yystate];
  if (yyn == YYPACT_NINF)
    goto yydefault;

  /* Not known => get a look-ahead token if don't already have one.  */

  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
  if (yychar == YYEMPTY)
    {
      YYDPRINTF ((stderr, "Reading a token: "));
      yychar = YYLEX;
    }

  if (yychar <= YYEOF)
    {
      yychar = yytoken = YYEOF;
      YYDPRINTF ((stderr, "Now at end of input.\n"));
    }
  else
    {
      yytoken = YYTRANSLATE (yychar);
      YY_SYMBOL_PRINT ("Next token is", yytoken, &yylval, &yylloc);
    }

  /* If the proper action on seeing token YYTOKEN is to reduce or to
     detect an error, take that action.  */
  yyn += yytoken;
  if (yyn < 0 || YYLAST < yyn || yycheck[yyn] != yytoken)
    goto yydefault;
  yyn = yytable[yyn];
  if (yyn <= 0)
    {
      if (yyn == 0 || yyn == YYTABLE_NINF)
	goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }

  if (yyn == YYFINAL)
    YYACCEPT;

  /* Count tokens shifted since error; after three, turn off error
     status.  */
  if (yyerrstatus)
    yyerrstatus--;

  /* Shift the look-ahead token.  */
  YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);

  /* Discard the shifted token unless it is eof.  */
  if (yychar != YYEOF)
    yychar = YYEMPTY;

  yystate = yyn;
  *++yyvsp = yylval;

  goto yynewstate;


/*-----------------------------------------------------------.
| yydefault -- do the default action for the current state.  |
`-----------------------------------------------------------*/
yydefault:
  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;
  goto yyreduce;


/*-----------------------------.
| yyreduce -- Do a reduction.  |
`-----------------------------*/
yyreduce:
  /* yyn is the number of a rule to reduce with.  */
  yylen = yyr2[yyn];

  /* If YYLEN is nonzero, implement the default value of the action:
     `$$ = $1'.

     Otherwise, the following line sets YYVAL to garbage.
     This behavior is undocumented and Bison
     users should not rely upon it.  Assigning to YYVAL
     unconditionally makes the parser a bit smaller, and it avoids a
     GCC warning that YYVAL may be used uninitialized.  */
  yyval = yyvsp[1-yylen];


  YY_REDUCE_PRINT (yyn);
  switch (yyn)
    {
        case 5:
#line 122 "par.y"
    { 
		if (cmd_parse_tree) {
			printtree((yyvsp[(1) - (1)].expr), 0); 
		}
		if (cmd_roll) {
			int i;
			struct symtab *st = NULL;
			set_symtab((yyvsp[(1) - (1)].expr), st);
			struct roll_value *rv = roll((yyvsp[(1) - (1)].expr));
			for (i = 0; i < rv->count; i++) {
				printf("%i ", rv->values[i]);
			}
			printf("\n");
			free_roll(rv);
		}
		if (cmd_calc) {
			struct symtab *st = NULL;
            (yyvsp[(1) - (1)].expr) = optimize((yyvsp[(1) - (1)].expr));
			set_symtab((yyvsp[(1) - (1)].expr), st);
			// we can use agnostic here as we are getting a scalar anyway.
			set_ordering((yyvsp[(1) - (1)].expr), agnostic);
			struct val_list *vl = eval((yyvsp[(1) - (1)].expr));
			struct val_list *cl = vl = list_sort(vl);
			while (cl) {
				if (cl->count != 1) {
					yyerror("result is not scalar");
					break;
				}
                // XXX make output resolution configurable
				printf("%3i\t%0.6f\t\n", cl->values[0], cl->prob);
				cl = cl->next;
			}
			list_free(vl);
		}
		if (cmd_rolleval) {
			int i;
			struct result_node *rl = NULL;
			for (i = 0; i < cmd_count; i++) {
				struct symtab *st = NULL;
				set_symtab((yyvsp[(1) - (1)].expr), st);
				struct roll_value *rv = roll((yyvsp[(1) - (1)].expr));
				if (rv->count != 1) {
					yyerror("Result not a single value");
					free_roll(rv);
					break;
				}
				result_add(&rl, rv->values[0]);
				free_roll(rv);
			}
			
			struct result_node *cl = rl;
			int sum = 0;
			while (cl != NULL) {
				sum += cl->prob;
				cl = cl->next;
			}
			cl = rl;
			while (cl != NULL) {
				float fprob;
				fprob = 1.0*cl->prob;
				fprob /= sum;
                // XXX make output resolution configurable
				printf("%3i\t%0.6f\n", cl->value, fprob);
				struct result_node *old = cl;
				cl = cl->next;
				free(old);
			}
		}
	;}
    break;

  case 6:
#line 193 "par.y"
    { (yyval.expr) = (yyvsp[(1) - (1)].expr); ;}
    break;

  case 7:
#line 194 "par.y"
    { (yyval.expr) = (yyvsp[(1) - (1)].expr); ;}
    break;

  case 8:
#line 197 "par.y"
    { (yyval.expr) = number_create((yyvsp[(1) - (1)].ival)); ;}
    break;

  case 9:
#line 198 "par.y"
    { (yyval.expr) = variable_create((yyvsp[(1) - (1)].tval)); ;}
    break;

  case 10:
#line 199 "par.y"
    { (yyval.expr) = (yyvsp[(2) - (3)].expr); ;}
    break;

  case 11:
#line 200 "par.y"
    { (yyval.expr) = negate_create((yyvsp[(2) - (2)].expr)); ;}
    break;

  case 12:
#line 201 "par.y"
    { (yyval.expr) = plus_create((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); ;}
    break;

  case 13:
#line 202 "par.y"
    { (yyval.expr) = minus_create((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); ;}
    break;

  case 14:
#line 203 "par.y"
    { (yyval.expr) = multi_create((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); ;}
    break;

  case 15:
#line 204 "par.y"
    { (yyval.expr) = divi_create((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); ;}
    break;

  case 16:
#line 205 "par.y"
    { (yyval.expr) = mod_create((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); ;}
    break;

  case 17:
#line 206 "par.y"
    { (yyval.expr) = expo_create((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); ;}
    break;

  case 18:
#line 207 "par.y"
    { (yyval.expr) = scat_create((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); ;}
    break;

  case 19:
#line 208 "par.y"
    { (yyval.expr) = dice_create((yyvsp[(2) - (2)].expr)); ;}
    break;

  case 20:
#line 209 "par.y"
    { (yyval.expr) = sum_create((yyvsp[(2) - (2)].expr)); ;}
    break;

  case 21:
#line 210 "par.y"
    { (yyval.expr) = prod_create((yyvsp[(2) - (2)].expr)); ;}
    break;

  case 22:
#line 211 "par.y"
    { (yyval.expr) = count_create((yyvsp[(2) - (2)].expr)); ;}
    break;

  case 23:
#line 214 "par.y"
    { (yyval.expr) = rep_create((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); ;}
    break;

  case 24:
#line 215 "par.y"
    { (yyval.expr) = (yyvsp[(2) - (3)].expr); ;}
    break;

  case 25:
#line 216 "par.y"
    { (yyval.expr) = elist_create(); ;}
    break;

  case 26:
#line 217 "par.y"
    { (yyval.expr) = range_create((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); ;}
    break;

  case 27:
#line 218 "par.y"
    { (yyval.expr) = lcat_create((yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr)); ;}
    break;

  case 28:
#line 219 "par.y"
    { (yyval.expr) = perm_create((yyvsp[(2) - (2)].expr)); ;}
    break;

  case 29:
#line 220 "par.y"
    { (yyval.expr) = sort_create((yyvsp[(2) - (2)].expr)); ;}
    break;

  case 30:
#line 221 "par.y"
    { (yyval.expr) = rev_create((yyvsp[(2) - (2)].expr)); ;}
    break;

  case 31:
#line 222 "par.y"
    { (yyval.expr) = (yyvsp[(1) - (1)].expr); ;}
    break;

  case 32:
#line 223 "par.y"
    { (yyval.expr) = ifthenelse_create((yyvsp[(2) - (6)].expr), (yyvsp[(4) - (6)].expr), (yyvsp[(6) - (6)].expr)); ;}
    break;

  case 33:
#line 224 "par.y"
    { (yyval.expr) = let_create((yyvsp[(4) - (6)].expr), (yyvsp[(6) - (6)].expr), (yyvsp[(2) - (6)].tval)); ;}
    break;

  case 34:
#line 225 "par.y"
    { (yyval.expr) = whiledo_create((yyvsp[(4) - (6)].expr), (yyvsp[(6) - (6)].expr), (yyvsp[(2) - (6)].tval)); ;}
    break;

  case 35:
#line 226 "par.y"
    { (yyval.expr) = foreach_create((yyvsp[(4) - (6)].expr), (yyvsp[(6) - (6)].expr), (yyvsp[(2) - (6)].tval)); ;}
    break;

  case 36:
#line 229 "par.y"
    { (yyval.expr) = first_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), drop); ;}
    break;

  case 37:
#line 230 "par.y"
    { (yyval.expr) = first_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), keep); ;}
    break;

  case 38:
#line 231 "par.y"
    { (yyval.expr) = first_create((yyvsp[(2) - (3)].expr), (yyvsp[(3) - (3)].expr), keep); ;}
    break;

  case 39:
#line 232 "par.y"
    { (yyval.expr) = last_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), drop); ;}
    break;

  case 40:
#line 233 "par.y"
    { (yyval.expr) = last_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), keep); ;}
    break;

  case 41:
#line 234 "par.y"
    { (yyval.expr) = last_create((yyvsp[(2) - (3)].expr), (yyvsp[(3) - (3)].expr), keep); ;}
    break;

  case 42:
#line 235 "par.y"
    { (yyval.expr) = high_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), drop); ;}
    break;

  case 43:
#line 236 "par.y"
    { (yyval.expr) = high_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), keep); ;}
    break;

  case 44:
#line 237 "par.y"
    { (yyval.expr) = high_create((yyvsp[(2) - (3)].expr), (yyvsp[(3) - (3)].expr), keep); ;}
    break;

  case 45:
#line 238 "par.y"
    { (yyval.expr) = low_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), drop); ;}
    break;

  case 46:
#line 239 "par.y"
    { (yyval.expr) = low_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), keep); ;}
    break;

  case 47:
#line 240 "par.y"
    { (yyval.expr) = low_create((yyvsp[(2) - (3)].expr), (yyvsp[(3) - (3)].expr), keep); ;}
    break;

  case 48:
#line 241 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), drop, eq); ;}
    break;

  case 49:
#line 242 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), keep, eq); ;}
    break;

  case 50:
#line 243 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(2) - (3)].expr), (yyvsp[(3) - (3)].expr), keep, eq); ;}
    break;

  case 51:
#line 244 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), drop, ne); ;}
    break;

  case 52:
#line 245 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), keep, ne); ;}
    break;

  case 53:
#line 246 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(2) - (3)].expr), (yyvsp[(3) - (3)].expr), keep, ne); ;}
    break;

  case 54:
#line 247 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), drop, gt); ;}
    break;

  case 55:
#line 248 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), keep, gt); ;}
    break;

  case 56:
#line 249 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(2) - (3)].expr), (yyvsp[(3) - (3)].expr), keep, gt); ;}
    break;

  case 57:
#line 250 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), drop, lt); ;}
    break;

  case 58:
#line 251 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), keep, lt); ;}
    break;

  case 59:
#line 252 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(2) - (3)].expr), (yyvsp[(3) - (3)].expr), keep, lt); ;}
    break;

  case 60:
#line 253 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), drop, ge); ;}
    break;

  case 61:
#line 254 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), keep, ge); ;}
    break;

  case 62:
#line 255 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(2) - (3)].expr), (yyvsp[(3) - (3)].expr), keep, ge); ;}
    break;

  case 63:
#line 256 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), drop, le); ;}
    break;

  case 64:
#line 257 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(3) - (4)].expr), (yyvsp[(4) - (4)].expr), keep, le); ;}
    break;

  case 65:
#line 258 "par.y"
    { (yyval.expr) = comparison_create((yyvsp[(2) - (3)].expr), (yyvsp[(3) - (3)].expr), keep, le); ;}
    break;


/* Line 1267 of yacc.c.  */
#line 1997 "par.c"
      default: break;
    }
  YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);

  YYPOPSTACK (yylen);
  yylen = 0;
  YY_STACK_PRINT (yyss, yyssp);

  *++yyvsp = yyval;


  /* Now `shift' the result of the reduction.  Determine what state
     that goes to, based on the state we popped back to and the rule
     number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTOKENS] + *yyssp;
  if (0 <= yystate && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTOKENS];

  goto yynewstate;


/*------------------------------------.
| yyerrlab -- here on detecting error |
`------------------------------------*/
yyerrlab:
  /* If not already recovering from an error, report this error.  */
  if (!yyerrstatus)
    {
      ++yynerrs;
#if ! YYERROR_VERBOSE
      yyerror (YY_("syntax error"));
#else
      {
	YYSIZE_T yysize = yysyntax_error (0, yystate, yychar);
	if (yymsg_alloc < yysize && yymsg_alloc < YYSTACK_ALLOC_MAXIMUM)
	  {
	    YYSIZE_T yyalloc = 2 * yysize;
	    if (! (yysize <= yyalloc && yyalloc <= YYSTACK_ALLOC_MAXIMUM))
	      yyalloc = YYSTACK_ALLOC_MAXIMUM;
	    if (yymsg != yymsgbuf)
	      YYSTACK_FREE (yymsg);
	    yymsg = (char *) YYSTACK_ALLOC (yyalloc);
	    if (yymsg)
	      yymsg_alloc = yyalloc;
	    else
	      {
		yymsg = yymsgbuf;
		yymsg_alloc = sizeof yymsgbuf;
	      }
	  }

	if (0 < yysize && yysize <= yymsg_alloc)
	  {
	    (void) yysyntax_error (yymsg, yystate, yychar);
	    yyerror (yymsg);
	  }
	else
	  {
	    yyerror (YY_("syntax error"));
	    if (yysize != 0)
	      goto yyexhaustedlab;
	  }
      }
#endif
    }



  if (yyerrstatus == 3)
    {
      /* If just tried and failed to reuse look-ahead token after an
	 error, discard it.  */

      if (yychar <= YYEOF)
	{
	  /* Return failure if at end of input.  */
	  if (yychar == YYEOF)
	    YYABORT;
	}
      else
	{
	  yydestruct ("Error: discarding",
		      yytoken, &yylval);
	  yychar = YYEMPTY;
	}
    }

  /* Else will try to reuse look-ahead token after shifting the error
     token.  */
  goto yyerrlab1;


/*---------------------------------------------------.
| yyerrorlab -- error raised explicitly by YYERROR.  |
`---------------------------------------------------*/
yyerrorlab:

  /* Pacify compilers like GCC when the user code never invokes
     YYERROR and the label yyerrorlab therefore never appears in user
     code.  */
  if (/*CONSTCOND*/ 0)
     goto yyerrorlab;

  /* Do not reclaim the symbols of the rule which action triggered
     this YYERROR.  */
  YYPOPSTACK (yylen);
  yylen = 0;
  YY_STACK_PRINT (yyss, yyssp);
  yystate = *yyssp;
  goto yyerrlab1;


/*-------------------------------------------------------------.
| yyerrlab1 -- common code for both syntax error and YYERROR.  |
`-------------------------------------------------------------*/
yyerrlab1:
  yyerrstatus = 3;	/* Each real token shifted decrements this.  */

  for (;;)
    {
      yyn = yypact[yystate];
      if (yyn != YYPACT_NINF)
	{
	  yyn += YYTERROR;
	  if (0 <= yyn && yyn <= YYLAST && yycheck[yyn] == YYTERROR)
	    {
	      yyn = yytable[yyn];
	      if (0 < yyn)
		break;
	    }
	}

      /* Pop the current state because it cannot handle the error token.  */
      if (yyssp == yyss)
	YYABORT;


      yydestruct ("Error: popping",
		  yystos[yystate], yyvsp);
      YYPOPSTACK (1);
      yystate = *yyssp;
      YY_STACK_PRINT (yyss, yyssp);
    }

  if (yyn == YYFINAL)
    YYACCEPT;

  *++yyvsp = yylval;


  /* Shift the error token.  */
  YY_SYMBOL_PRINT ("Shifting", yystos[yyn], yyvsp, yylsp);

  yystate = yyn;
  goto yynewstate;


/*-------------------------------------.
| yyacceptlab -- YYACCEPT comes here.  |
`-------------------------------------*/
yyacceptlab:
  yyresult = 0;
  goto yyreturn;

/*-----------------------------------.
| yyabortlab -- YYABORT comes here.  |
`-----------------------------------*/
yyabortlab:
  yyresult = 1;
  goto yyreturn;

#ifndef yyoverflow
/*-------------------------------------------------.
| yyexhaustedlab -- memory exhaustion comes here.  |
`-------------------------------------------------*/
yyexhaustedlab:
  yyerror (YY_("memory exhausted"));
  yyresult = 2;
  /* Fall through.  */
#endif

yyreturn:
  if (yychar != YYEOF && yychar != YYEMPTY)
     yydestruct ("Cleanup: discarding lookahead",
		 yytoken, &yylval);
  /* Do not reclaim the symbols of the rule which action triggered
     this YYABORT or YYACCEPT.  */
  YYPOPSTACK (yylen);
  YY_STACK_PRINT (yyss, yyssp);
  while (yyssp != yyss)
    {
      yydestruct ("Cleanup: popping",
		  yystos[*yyssp], yyvsp);
      YYPOPSTACK (1);
    }
#ifndef yyoverflow
  if (yyss != yyssa)
    YYSTACK_FREE (yyss);
#endif
#if YYERROR_VERBOSE
  if (yymsg != yymsgbuf)
    YYSTACK_FREE (yymsg);
#endif
  /* Make sure YYID is used.  */
  return YYID (yyresult);
}


#line 260 "par.y"


int yyerror(char *s) {
	fprintf(stderr, "%s:%li %s\n", yycurrfilename(), yycurrlinenum(), s);
	return (1);
}

extern int yylineno;

long yycurrlinenum() {
	return yylineno;
}

char *yycurrfilename() {
	return filename;
}

void yynodefailed() {
	exit(1);
}

